/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_interrupt.c
* Description : The interrupt handler
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Interrupt vector define
******************************************************************************/
#pragma interrupt mtr_hall_interrupt(vect=INTP1)
#pragma interrupt mtr_hall_interrupt(vect=INTP2)
#pragma interrupt mtr_hall_interrupt(vect=INTP3)
#pragma interrupt mtr_alarm_interrupt(vect=INTP4)
#pragma interrupt mtr_tau0_interrupt(vect=INTTM00)
#pragma interrupt mtr_carrier_interrupt(vect=INTTRD0)

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_spm_hall_120_cpm.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

#ifdef ICS_USE
/****** for ICS ******/
#include "ics2_RL78G1F.h"
/*********************/
#endif

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile uint16_t   g_u2_cnt_boot_mode;                          /* counter for boot mode time */
extern volatile uint16_t   g_u2_cnt_wait_stop;                          /* counter for waiting for motor stop */
extern volatile uint8_t    g_u1_flg_wait_stop;                          /* flag for waiting for motor stop */

extern volatile uint8_t    g_u1_enable_write;                           /* ICS write enable flag */

extern volatile int16_t    g_s2_vdc_ad;                                 /* bus voltage : scale(2^7) */
extern volatile int16_t    g_s2_pdrv_ad;                                /* predriver voltage : scale(2^0) */

#ifdef ICS_USE
/****** for ICS ******/
extern volatile uint8_t    g_u1_cnt_ics;                                /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
extern volatile uint16_t   g_u2_run_mode;                               /* run mode */
extern volatile uint8_t    g_u1_mode_system;                            /* system mode */
extern volatile uint8_t    g_u1_error_status;                           /* error status */

/******************************************************************************
* Function Name : mtr_hall_interrupt
* Description   : Hall sensor interrupt(INTP1, 2, 3)
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_hall_interrupt(void)
{
    EI();

    if (g_u1_flg_wait_stop == MTR_FLG_SET)                              /* waiting for motor stop */
    {
        g_u2_cnt_wait_stop = 0;
    }
    else
    {
        mtr_pattern_set();                                              /* set voltage pattern */
    }

    DI();
}

/******************************************************************************
* Function Name : mtr_alarm_interrupt
* Description   : Alarm interrupt(INTP4)
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_alarm_interrupt(void)
{
#ifdef HW_ERR_DET_ON
    R_MTR_ExecEvent(MTR_EVENT_ERROR);                                   /* execute error event */

    DI();

    g_u1_error_status |= MTR_ALARM_ERROR;                               /* error status update : alarm error */

    EI();

    mtr_clear_oc_flag();                                                /* clear forced cutoff flag (detect alarm) */
#endif
}

/******************************************************************************
* Function Name : mtr_tau0_interrupt
* Description   : TAU0 interrupt (period: 500[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_tau0_interrupt(void)
{
    EI();

    /*==============================*/
    /*     count boot mode time     */
    /*==============================*/

    if (g_u1_flg_wait_stop == MTR_FLG_CLR)                              /* check wait stop flag */
    {
        if (g_u1_mode_system == MTR_MODE_RUN)                           /* check system mode */
        {
            if (g_u2_run_mode == MTR_BOOT_MODE)
            {
                g_u2_cnt_boot_mode++;

                if (g_u2_cnt_boot_mode >= MTR_START_CNT)
                {
                    g_u2_cnt_boot_mode = 0;
                    g_u2_run_mode = MTR_HALL_120_MODE;                  /* normal control mode */
                }
            }
        }
    }

    DI();
}

/******************************************************************************
* Function Name : mtr_carrier_interrupt
* Description   : Carrier interrupt (period: 50[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_carrier_interrupt(void)
{
    int16_t s2_temp;

    EI();

    /*==============================*/
    /*           set vdc            */
    /*==============================*/

    DI();

    s2_temp = mtr_get_adc(MTR_ADCCH_VM);

    EI();

    g_s2_vdc_ad = R_FIX8_mul_int16(s2_temp, MTR_VDC_SCALING);           /* g_s2_vdc_ad : scale(2^7) */

    /*==============================*/
    /*           set pdrv           */
    /*==============================*/

    DI();

    g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

    EI();

    /*============================*/
    /*      check error           */
    /*============================*/

    mtr_error_check();                                                  /* check error status */

    /*** wait motor stop ***/
    if (g_u1_flg_wait_stop == MTR_FLG_SET)                              /* waiting for motor stop */
    {
        g_u2_cnt_wait_stop++;

        if (g_u2_cnt_wait_stop >= MTR_STOP_WAIT_CNT)
        {
            g_u1_flg_wait_stop = MTR_FLG_CLR;
            g_u2_cnt_wait_stop = 0;
        }
    }

#ifdef ICS_USE
    /****** for ICS ******/
    g_u1_cnt_ics++;

    if (g_u1_cnt_ics > MTR_ICS_DECIMATION)                              /* period : 250[us] */
    {
        DI();                                                           /* interrupt disable */

        ics2_watchpoint();

        EI();                                                           /* interrupt enable */

        g_u1_cnt_ics = 0;
    }
    /*********************/
#endif

    /*** set variables ***/
    if (MTR_FLG_SET == g_u1_enable_write)
    {
        mtr_set_variables();
        g_u1_enable_write = MTR_FLG_CLR;
    }

    mtr_clear_trd0_imfa();                                              /* clear compare match flag A */

    DI();
}

