/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_spm_hall_120_cpm.c
* Description : The processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_spm_hall_120_cpm.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/******************************************************************************
* Exported global variables (to be accessed by other files)
******************************************************************************/
volatile uint16_t   g_u2_cnt_boot_mode;                                 /* counter for boot mode time */
volatile uint16_t   g_u2_cnt_wait_stop;                                 /* counter for waiting for motor stop */
volatile uint8_t    g_u1_flg_wait_stop;                                 /* flag for waiting for motor stop */

volatile uint8_t    g_u1_enable_write;                                  /* ICS write enable flag */

volatile int16_t    g_s2_vdc_ad;                                        /* bus voltage : scale(2^7) */
volatile int16_t    g_s2_reci_vdc_ad;                                   /* reciprocal bus voltage : scale(2^15) */
volatile int16_t    g_s2_pdrv_ad;                                       /* predriver voltage : scale(2^0) */

volatile int16_t    g_s2_pwm_duty;                                      /* PWM duty : scale(2^0) */
volatile int16_t    g_s2_ref_speed_rad;                                 /* motor speed reference : scale(2^2) */
volatile int16_t    g_s2_speed_rad;                                     /* motor speed : scale(2^2) */
volatile int16_t    g_s2_speed_lpf_k;                                   /* speed LPF parameter : scale(2^14) */

#ifdef ICS_USE
/****** for ICS ******/
volatile uint8_t    g_u1_cnt_ics;                                       /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
volatile uint16_t   g_u2_run_mode;                                      /* run mode */
volatile uint8_t    g_u1_error_status;                                  /* error status */
volatile uint8_t    g_u1_mode_system;                                   /* system mode */

/******************************************************************************
* Private global variables and functions
******************************************************************************/
volatile uint8_t    g_u1_hall_signal;                                   /* hall value of 3 phase */
volatile uint8_t    g_u1_v_pattern;                                     /* voltage pattern */
volatile uint16_t   g_u2_cnt_timeout;                                   /* counter for timeout error */
volatile uint8_t    g_u1_direction;                                     /* rotational direction (0:CW ,1:CCW) */
volatile uint16_t   g_u2_hall_timer_cnt;                                /* value of timer counter per hall */
volatile uint16_t   g_u2_pre_hall_timer_cnt;                            /* previous value of timer counter per hall */
volatile int16_t    g_s2_timer_cnt_ave;                                 /* counts for 360 degrees */
volatile uint16_t   g_u2_timer_cnt_buf[MTR_TIMER_CNT_BUF_NUM];          /* counts for 60 degrees */
volatile uint16_t   g_u2_timer_cnt_num;                                 /* array element number before 360 degrees */

volatile MTR_ICS_INPUT ics_input_buff;                                  /* struct for ICS input */

static void mtr_speed_calc(void);
static void mtr_start_init(void);

/* for sequence control */
static uint8_t mtr_act_run(uint8_t);
static uint8_t mtr_act_stop(uint8_t);
static uint8_t mtr_act_reset(uint8_t);
static uint8_t mtr_act_error(uint8_t);
static uint8_t mtr_act_none(uint8_t);

static const volatile uint8_t g_u1_def_state[MTR_SIZE_EVENT][MTR_SIZE_STATE] = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ MTR_MODE_STOP,   MTR_MODE_STOP,   MTR_MODE_ERROR },
/* 1:MTR_EVENT_RUN   */{ MTR_MODE_RUN,    MTR_MODE_RUN,    MTR_MODE_ERROR },
/* 2:MTR_EVENT_ERROR */{ MTR_MODE_ERROR,  MTR_MODE_ERROR,  MTR_MODE_ERROR },
/* 3:MTR_EVENT_RESET */{ MTR_MODE_STOP,   MTR_MODE_ERROR,  MTR_MODE_STOP  },};

static uint8_t (* gp_u1_def_action[MTR_SIZE_EVENT][MTR_SIZE_STATE])(uint8_t u1_state) = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ mtr_act_none,    mtr_act_stop,    mtr_act_none  },
/* 1:MTR_EVENT_RUN   */{ mtr_act_run,     mtr_act_none,    mtr_act_none  },
/* 2:MTR_EVENT_ERROR */{ mtr_act_error,   mtr_act_error,   mtr_act_none  },
/* 3:MTR_EVENT_RESET */{ mtr_act_reset,   mtr_act_error,   mtr_act_reset },};

/******************************************************************************
* Function Name : R_MTR_InitSequence
* Description   : Initialize variables for sequence control
* Arguments     : none
* Return Value  : none
******************************************************************************/
void R_MTR_InitSequence(void)
{
    g_u1_mode_system = MTR_MODE_STOP;                                   /* system mode update : stop mode */
}

/******************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Change motor state and select action
* Arguments     : u1_event - event
* Return Value  : none
******************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event)
{
    uint8_t u1_mode_system;

    u1_mode_system = g_u1_mode_system;

#ifdef FW_ERR_DET_UNKNOWN
    if (u1_event >= MTR_SIZE_EVENT)                                     /* check unknown event */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else if (u1_mode_system >= MTR_SIZE_STATE)                          /* check unknown state */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else
    {
        /* change motor state and select action */
        g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
    }
#else
    /* change motor state and select action */
    g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
#endif
}

/******************************************************************************
* Function Name : mtr_act_run
* Description   : Processing for motor control start
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_run(uint8_t u1_state)
{
    g_u1_flg_wait_stop = MTR_FLG_SET;

    mtr_start_init();                                                   /* initialize variables when motor control start */

    mtr_ctrl_start();                                                   /* hall sensor interrupt enable for motor control start */

    while (g_u1_flg_wait_stop == MTR_FLG_SET);                          /* waiting for motor stop */

    mtr_pattern_set();                                                  /* set initial voltage pattern */

    if (g_u1_mode_system == MTR_MODE_ERROR)                             /* check system mode */
    {
        u1_state = MTR_MODE_ERROR;                                      /* system mode update : error mode */
    }

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_stop
* Description   : Processing for motor control stop
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_stop(uint8_t u1_state)
{
    g_u2_run_mode = MTR_BOOT_MODE;                                      /* boot mode */

    mtr_ctrl_stop();                                                    /* TRD output disable, hall sensor interrupt disable, */
                                                                        /* motor output port is on/off for motor control stop */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_none
* Description   : Do Nothing
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_none(uint8_t u1_state)
{
    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_reset
* Description   : Processing for motor control reset
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_reset(uint8_t u1_state)
{
    g_u2_cnt_boot_mode = 0;
    g_u2_cnt_wait_stop = 0;
    g_u1_flg_wait_stop = MTR_FLG_CLR;

    g_u1_enable_write = 0;

    g_s2_vdc_ad = 0;
    g_s2_reci_vdc_ad = 0;
    g_s2_pdrv_ad = 0;

    g_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;
    g_s2_ref_speed_rad = 0;
    g_s2_speed_rad = 0;
    g_s2_speed_lpf_k = MTR_SPEED_LPF_K;                                 /* set MTR_SPEED_LPF_K */

#ifdef ICS_USE
/****** for ICS ******/
    g_u1_cnt_ics = 0;
/*********************/
#endif

    g_u2_run_mode = MTR_BOOT_MODE;                                      /* set MTR_BOOT_MODE */
    g_u1_error_status = 0;
    g_u1_mode_system = MTR_MODE_STOP;                                   /* set MTR_MODE_STOP */

    g_u1_hall_signal = 0;
    g_u1_v_pattern = 0;
    g_u2_cnt_timeout = 0;
    g_u1_direction = MTR_CW;                                            /* set MTR_CW */
    g_u2_hall_timer_cnt = 0;
    g_u2_pre_hall_timer_cnt = 0;
    g_s2_timer_cnt_ave = 0;
    g_u2_timer_cnt_buf[0] = 0;
    g_u2_timer_cnt_buf[1] = 0;
    g_u2_timer_cnt_buf[2] = 0;
    g_u2_timer_cnt_buf[3] = 0;
    g_u2_timer_cnt_buf[4] = 0;
    g_u2_timer_cnt_buf[5] = 0;
    g_u2_timer_cnt_num = 0;

    ics_input_buff.s2_ref_speed = 0;
    ics_input_buff.s2_direction = 0;
    ics_input_buff.s2_speed_lpf_k = 0;

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_error
* Description   : Processing when detecting an error
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_error(uint8_t u1_state)
{
    g_u2_run_mode = MTR_BOOT_MODE;                                      /* boot mode */

    mtr_ctrl_stop();                                                    /* TRD output disable, hall sensor interrupt disable, */
                                                                        /* motor output port is on/off for motor control stop */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_pattern_set
* Description   : Set voltage pattern
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_pattern_set(void)
{
    g_u2_cnt_timeout = 0;                                               /* clear counter for timeout error */

    mtr_speed_calc();                                                   /* speed calculation */

    /*** get hall signal ***/
    g_u1_hall_signal  =  MTR_PORT_HALL_U << 2;
    g_u1_hall_signal |= (MTR_PORT_HALL_V << 1);
    g_u1_hall_signal |=  MTR_PORT_HALL_W;

    /*** set voltage pattern ***/
    /*** complementary first half 60 degree chopping ***/
    if (g_u1_direction == MTR_CW)                                       /* check rotational direction */
    {
        switch (g_u1_hall_signal)
        {
            case MTR_PATTERN_CW_W_V:                                    /* from W phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_UN_ON;
            break;

            case MTR_PATTERN_CW_U_V:                                    /* from U phase to V phase */
                g_u1_v_pattern = MTR_WP_ON_V_PWM;
            break;

            case MTR_PATTERN_CW_U_W:                                    /* from U phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_VN_ON;
            break;

            case MTR_PATTERN_CW_V_W:                                    /* from V phase to W phase */
                g_u1_v_pattern = MTR_UP_ON_W_PWM;
            break;

            case MTR_PATTERN_CW_V_U:                                    /* from V phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_WN_ON;
            break;

            case MTR_PATTERN_CW_W_U:                                    /* from W phase to U phase */
                g_u1_v_pattern = MTR_VP_ON_U_PWM;
            break;

            default:                                                    /* hall signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else if (g_u1_direction == MTR_CCW)                                 /* check rotational direction */
    {
        switch (g_u1_hall_signal)
        {
            case MTR_PATTERN_CCW_W_V:                                   /* from W phase to V phase */
                g_u1_v_pattern = MTR_WP_ON_U_PWM;
            break;

            case MTR_PATTERN_CCW_W_U:                                   /* from W phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_UN_ON;
            break;

            case MTR_PATTERN_CCW_V_U:                                   /* from V phase to U phase */
                g_u1_v_pattern = MTR_VP_ON_W_PWM;
            break;

            case MTR_PATTERN_CCW_V_W:                                   /* from V phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_WN_ON;
            break;

            case MTR_PATTERN_CCW_U_W:                                   /* from U phase to W phase */
                g_u1_v_pattern = MTR_UP_ON_V_PWM;
            break;

            case MTR_PATTERN_CCW_U_V:                                   /* from U phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_VN_ON;
            break;

            default:                                                    /* hall signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else
    {
        /* Do Nothing */
    }

    if (g_u1_mode_system != MTR_MODE_ERROR)                             /* check system mode */
    {
        mtr_change_pattern(g_u1_v_pattern);                             /* change voltage pattern */
    }
}

/******************************************************************************
* Function Name : mtr_speed_calc
* Description   : Calculate motor speed
* Arguments     : none
* Return Value  : none
******************************************************************************/
static void mtr_speed_calc(void)
{
    int16_t  s2_temp;
    uint16_t u2_temp;

    g_u2_hall_timer_cnt = MTR_TAU1_CNT;                                 /* get value of timer counter (TCR01) */

    u2_temp = (g_u2_pre_hall_timer_cnt - g_u2_hall_timer_cnt);          /* calculate timer count in 60 degrees */

    if (g_u2_run_mode != MTR_BOOT_MODE)                                 /* check run mode */
    {   /*** average 6 times of motor speed timer ***/
        g_s2_timer_cnt_ave -= g_u2_timer_cnt_buf[g_u2_timer_cnt_num];
        g_u2_timer_cnt_buf[g_u2_timer_cnt_num] = u2_temp;
        g_s2_timer_cnt_ave += g_u2_timer_cnt_buf[g_u2_timer_cnt_num];

        g_u2_timer_cnt_num++;

        if (g_u2_timer_cnt_num > (MTR_TIMER_CNT_BUF_NUM - 1))
        {
            g_u2_timer_cnt_num = 0;
        }

        /* *** low speed limit *** */
        if (g_s2_timer_cnt_ave < 0)
        {
            g_s2_timer_cnt_ave = 0x7FFF;
        }

        if (g_u2_timer_cnt_buf[MTR_TIMER_CNT_BUF_NUM - 1] != 0)
        {
            /*** check overflow and calculate RPM & low pass filter ***/
            if (g_s2_timer_cnt_ave > MTR_SPEED_CALC_BASE)
            {
                s2_temp = R_FIX_div_int16(MTR_SPEED_CALC_BASE, g_s2_timer_cnt_ave);
                s2_temp = s2_temp - g_s2_speed_rad;
                g_s2_speed_rad += R_FIX14_mul_int16(g_s2_speed_lpf_k, s2_temp);
            }
#ifdef FW_ERR_DET_OVERSPEED
            else
            {
                if (g_u2_run_mode == MTR_HALL_120_MODE)                 /* check run mode */
                {
                    R_MTR_ExecEvent(MTR_EVENT_ERROR);                   /* execute error event */

                    DI();

                    g_u1_error_status |= MTR_OVER_SPEED_ERROR;          /* error status update : over speed error */

                    EI();
                }
            }
#endif
        }
    }

    g_u2_pre_hall_timer_cnt = g_u2_hall_timer_cnt;                      /* keep timer counter value */
}

/******************************************************************************
* Function Name : mtr_start_init
* Description   : Initialize variables when motor control start
* Arguments     : none
* Return Value  : none
******************************************************************************/
static void mtr_start_init(void)
{
    int16_t s2_temp;

    g_u2_cnt_boot_mode = 0;
    g_u2_cnt_wait_stop = 0;
    g_u1_flg_wait_stop = MTR_FLG_CLR;

    DI();

    s2_temp = mtr_get_adc(MTR_ADCCH_VM);

    EI();

    g_s2_vdc_ad = R_FIX8_mul_int16(s2_temp, MTR_VDC_SCALING);
    g_s2_reci_vdc_ad = R_FIX_div_int16(MTR_RECIVDC_SCALING, g_s2_vdc_ad);

    /* duty 30% */
    g_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;

    g_s2_speed_rad = 0;

    g_u1_hall_signal = 0;
    g_u2_cnt_timeout = 0;
    g_u2_hall_timer_cnt = 0;
    g_u2_pre_hall_timer_cnt = 0;
    g_s2_timer_cnt_ave = 0;
    g_u2_timer_cnt_buf[0] = 0;
    g_u2_timer_cnt_buf[1] = 0;
    g_u2_timer_cnt_buf[2] = 0;
    g_u2_timer_cnt_buf[3] = 0;
    g_u2_timer_cnt_buf[4] = 0;
    g_u2_timer_cnt_buf[5] = 0;
    g_u2_timer_cnt_num = 0;
}

/******************************************************************************
* Function Name : mtr_set_variables
* Description   : Set variables (conversion of unit)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_set_variables(void)
{
    int16_t s2_temp;

    /* reference speed */
    s2_temp = R_FIX12_mul_int16(MTR_RPM_RAD, ics_input_buff.s2_ref_speed);
    g_s2_ref_speed_rad = s2_temp * MTR_POLE_PAIRS;                      /* [rpm]->[rad/s] */

    /* rotation direction */
    g_u1_direction = (uint8_t)ics_input_buff.s2_direction;

    /* speed lpf parameter */
    g_s2_speed_lpf_k = ics_input_buff.s2_speed_lpf_k;
}

/******************************************************************************
* Function Name : R_MTR_IcsInput
* Description   : Set value to buffer
* Arguments     : ICS input structure
* Return Value  : none
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *ics_input)
{
    /* reference speed */
    ics_input_buff.s2_ref_speed = ics_input->s2_ref_speed;

    /* rotation direction */
    ics_input_buff.s2_direction = ics_input->s2_direction;

    /* speed lpf parameter */
    ics_input_buff.s2_speed_lpf_k = ics_input->s2_speed_lpf_k;

    g_u1_enable_write = MTR_FLG_SET;
}

/******************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - Speed reference[rpm]
* Return Value  : none
******************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed)
{
    int16_t s2_temp;

    s2_temp = R_FIX12_mul_int16(MTR_RPM_RAD, ref_speed);
    g_s2_ref_speed_rad = s2_temp * MTR_POLE_PAIRS;
}

/******************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotational direction
* Arguments     : dir - Rotational direction
* Return Value  : none
******************************************************************************/
void R_MTR_SetDir(uint8_t dir)
{
    g_u1_direction = dir;
}

/******************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : none
* Return Value  : s2_speed_rpm - Speed
******************************************************************************/
int16_t R_MTR_GetSpeed(void)
{
    int16_t s2_speed_rpm;

    s2_speed_rpm = R_FIX10_mul_int16(MTR_RAD_RPM, g_s2_speed_rad);

    return (s2_speed_rpm);
}

/******************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : none
* Return Value  : g_u1_direction - Rotational direction
******************************************************************************/
uint8_t R_MTR_GetDir(void)
{
    return (g_u1_direction);
}

/******************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : none
* Return Value  : g_u1_mode_system - Motor status
******************************************************************************/
uint8_t R_MTR_GetStatus(void)
{
    return (g_u1_mode_system);
}

/******************************************************************************
* Function Name : mtr_error_check
* Description   : Check error status
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_error_check(void)
{
    /*==================================*/
    /*     over voltage error check     */
    /*==================================*/

#ifdef FW_ERR_DET_OVERVOLTAGE
    if (g_s2_vdc_ad > MTR_OVERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_VOLTAGE_ERROR;                    /* error status update : over voltage error */

        EI();
    }
#endif

    /*===================================*/
    /*     under voltage error check     */
    /*===================================*/

#ifdef FW_ERR_DET_UNDERVOLTAGE
    if (g_s2_vdc_ad < MTR_UNDERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_UNDER_VOLTAGE_ERROR;                   /* under voltage error */

        EI();
    }
#endif

    /*================================*/
    /*     over speed error check     */
    /*================================*/

#ifdef FW_ERR_DET_OVERSPEED
    if (g_s2_speed_rad > MTR_SPEED_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_SPEED_ERROR;                      /* error status update : over speed error */

        EI();
    }
#endif

    /*=========================================================*/
    /*     timeout error check (undetected hall interrupt)     */
    /*=========================================================*/

#ifdef FW_ERR_DET_TIMEOUT
    if (g_u2_run_mode == MTR_HALL_120_MODE)                             /* check run mode */
    {
        if (g_u1_mode_system == MTR_MODE_RUN)                           /* check system mode */
        {
            g_u2_cnt_timeout++;
        }
        else
        {
            g_u2_cnt_timeout = 0;
        }

        if (g_u2_cnt_timeout > MTR_TIMEOUT_CNT)                         /* check undetected time */
        {
            R_MTR_ExecEvent(MTR_EVENT_ERROR);                           /* execute error event */

            DI();

            g_u1_error_status |= MTR_TIMEOUT_ERROR;                     /* error status update : timeout error */

            EI();
        }
    }
#endif
}

